
# q-expansion of newform 13.4.a.b, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 13
weight = 4
poly_data = [-4, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 1], [4, -3], [-2, 1], [-10, 11], [34, 12], [-13, 0], [18, -17], [-26, -32], [104, -12], [-70, 96], [-26, -34], [102, 5], [-126, 22], [72, 143], [278, -121], [-74, 30], [-246, 124], [-434, -190], [150, -232], [50, -231], [98, 260], [-524, 40], [1070, -182], [-166, -388], [-718, 508], [558, 142], [-152, -304], [1216, -140], [782, -43], [1246, -236], [328, -344], [604, 567], [-2326, 202], [-676, 847], [-2134, -244], [-2106, 441], [1802, -924], [2610, -430], [-1718, 902], [-1642, -1114], [-80, -29], [-358, 558], [2916, -874], [794, 206], [-2306, 1395], [680, 670], [1284, -537], [-3214, 1737], [694, 1018], [470, -101], [1334, -621], [-1006, -993], [1466, -2536], [-3516, 1092], [894, 151], [3996, 480], [834, 2190], [-874, -1193], [2830, -3402], [2290, 484], [-5316, 1056], [-1682, -1487], [-2830, -1748], [584, 2858], [1434, 3345], [378, -2580], [-4322, -122], [-1686, 1841], [7332, -1753], [6518, 409], [-6566, 1650], [-2670, 54], [4380, 858], [-2246, 2010], [-2834, 378], [3866, 2721], [1730, -4392], [-8326, -696], [3442, -530], [4746, -5646], [-3988, 6781], [6262, -6139], [8482, -1417], [-13098, 1909], [2616, -2190], [10028, -111], [-3830, 1276], [8082, 2216], [7734, 1669], [9194, 1734], [11404, -1008], [-9146, -2137], [-7058, -1070], [5376, -6329], [2134, 178], [-5812, -4394], [-726, -112], [-11030, -1303], [-4488, -4580], [2830, 5763], [7172, 1301], [1534, -797], [1744, -2025], [3262, 8085], [1648, -3555], [-4966, 4152], [-2994, 1566], [-21510, 4416], [-1772, 5578], [5634, 1559], [9572, -1134], [194, -9088], [14858, -4610], [11226, -1534], [-20666, -1235], [23590, -3976], [-12290, 6848], [-7288, 11282], [-1918, -7682], [1360, -7028], [2298, 9636], [-4238, 6007], [7254, -1538], [19418, 1986], [-6778, -5846], [7954, 10266], [-10558, 1964], [8656, -4854], [22248, -1238], [4926, 1199], [5986, 8978], [-17278, -729], [-21152, -1704], [-6426, -4218], [15762, -7068], [-5886, -474], [12606, -131], [18738, -5978], [14234, -10156], [-7958, 10489], [16702, -1522], [8574, 4201], [-5584, 1350], [-5606, -540], [46, -12764], [-20550, 6446], [6590, 595], [17242, -15436], [-8920, 16580], [-10542, -1397], [-25946, 9371], [6190, -2725], [-2552, -13307], [-30020, -4164], [-22952, 4875], [-33844, -690], [13260, -4756], [49458, 2450], [-12246, 3532], [-24546, 13027], [20698, 2964], [-2514, -3859], [-33094, 2509], [30852, 7249], [19058, -7136], [-21818, 10859], [-13828, 8902], [-19586, 18784]]
